/* #undef ENABLE_BITMAP_ALLOCATOR */
/* #undef ENABLE_BOOST_FAST_POOL_ALLOCATOR */
/* #undef ENABLE_BOOST_POOL_ALLOCATOR */
/* #undef ENABLE_DEBUG_ALLOCATOR */
/* #undef ENABLE_MT_ALLOCATOR */
/* #undef ENABLE_NEW_ALLOCATOR */
/* #undef ENABLE_POOL_ALLOCATOR */
/* #undef ENABLE_TBB_ALLOCATOR */
/* #undef HAVE_ALLOCATOR_CONFIG */
